<?php

/*
  --------------------------------------------------------------------------
  Earth Perihelion and Aphelion Calculator

  This program computes the Earth's perihelion and aphelion for any
  given year from 1600 to 2200.

  Based on the NASA/JPL DE405 ephemeris model.

  Author:
  Jay Tanner - 2019

  Current Web Site
  www.NeoProgrammics.com

  Development Language:
  PHP v7.1.8

  License:
  This program source code and any related data and graphics are released
  under the provisions of the Creative Commons Attribution-ShareAlike 4.0
  International.

  https://creativecommons.org/licenses/by-sa/4.0/legalcode
  --------------------------------------------------------------------------
*/


   ob_start();

// Attach essential module(s).
//   include_once('PHPSL_TRANSPORT_API.php');

// Set GLOBAL Delta T mode to 'ON' state.
// Needed for Times Table.
   $DTOnOff = 'ON';

// Define the program cookie name and set it to expire in 30 days.
// This cookie can be shared by other programs in the same common
// home folder using this interface in common.
   $CookieName = 'Earth_Perihelion_and_Aphelion_Calculator_Settings';
   $SetToExpireIn30Days = time() + 30*86400;

// Set minimum and maximum year span to allow.   Any year outside this
// range will be considered an invalid year.   For the DE405 ephemeris,
// the valid year range is from 1600 to 2200 on the Gregorian calendar.
   $YMin = 1600;
   $YMax = 2200;

   $cYear = gmdate('Y');

// Define main TextArea text and background colors.
   $TxColor = 'black';
   $BgColor = 'white';

// Define script path and filename.
   $_AUTHOR_          = 'Jay Tanner';
   $_PROGRAM_VERSION_ = ''; $at = "&#97;&#116;"; $UTC = "&#85;&#84;&#67";
   $_SCRIPT_PATH_     = filter_input(INPUT_SERVER, 'SCRIPT_FILENAME');
   $_RUN_THIS_SCRIPT_ = filter_input(INPUT_POST, 'SCRIPT_NAME');

// Define internal document page title, HTML page heading text and revision date
   $_INTERNAL_TITLE_  = "Perihelion and Aphelion";
   $_INTERFACE_TITLE_ = "<b style='font-size:18px;'>Earth Perihelions and Aphelions Calculator </b><span style='font-size:14px;'><br>For Any Year within the 600-Year Period From $YMin to $YMax</span><br>Based on the NASA/JPL DE405 Ephemeris Model";
   $_REVISION_DATE_   = 'Revised: '. gmdate("l, F d, Y $at h:i A $UTC", FileMTime($_SCRIPT_PATH_));

// Do this only if [COMPUTE] button was clicked.
   $w = filter_input(INPUT_POST, 'ComputeButton');
   if (!isset($w))

// ----------------------------------------------------
// If [COMPUTE] button was clicked and an active cookie
// exists, then restore the interface settings from it.
     {
      $w = filter_input(INPUT_COOKIE, $CookieName);
   if (isset($w))
      {
       $CookieDataString = filter_input(INPUT_COOKIE, $CookieName);
        list
       (
        $Y,
        $mn,
        $dn,
        $hhmmssLT,
        $hhmmTZ,
        $LatLon,
        $elev
       ) = preg_split("[\|]", $CookieDataString);

        $elev = floatval(trim($elev));

        $m = intval($mn);
        $d = intval($dn);
        $LatLon = Check_Lat_Lon($LatLon);
        list($lat, $lon) = preg_split("[,]", $LatLon);
        $lat  = trim($lat);
        $lon  = trim($lon);
        $LatLon = "$lat  ,  $lon";
        $elevFt = ($elev * 0.3048);
        $elev = sprintf("%+1.1f", $elev);
      }

   else

// Set the default interface startup values.

 {
  $Y = date('Y');
  $m = date('n'); $mn  = sprintf("%02d", $m);
  $d = date('j'); $dn  = sprintf("%02d", $d);

  $hhmmssLT = date('H:i:s');

  $hhmmTZ   = substr(Hours_to_HMS(date('Z')/3600, 0, '+', ':'),0,6);

// Home location coordinates for Ovid, New York, USA.
  $LatLon = '+42 40 35.3  ,  -76 49 22.8';
  $LatLon = Check_Lat_Lon($LatLon);
  list($lat, $lon) = preg_split("[,]", $LatLon);
  $lat    = trim($lat);
  $lon    = trim($lon);
  $LatLon = "$lat  ,  $lon";

  $elev   = '+0.0 m';
  $elevFt = '+0.0 ft';

// Store interface settings in cookie.
   $CookieDataString = "$Y|$mn|$dn|$hhmmssLT|$hhmmTZ|$LatLon|$elev";
   setcookie ($CookieName, $CookieDataString, $SetToExpireIn30Days);
  }
      } // End of  if (!isset(_POST['ComputeButton']))


// Read values of all interface settings.
   $w = filter_input(INPUT_POST, 'ComputeButton');
   if (isset($w))
{
   $Y  = filter_input(INPUT_POST, 'Y');
   if(trim($Y) == ''){$Y = date('Y');}
   $mn = filter_input(INPUT_POST, 'mn'); $m  = intval($mn);
   $dn = filter_input(INPUT_POST, 'dn'); $d  = intval($dn);

   $hhmmssLT = filter_input(INPUT_POST, 'hhmmssLT');
   $hhmmssLT = Num_to_HMS ($hhmmssLT);

   $hhmmTZ = trim(filter_input(INPUT_POST, 'hhmmTZ'));
   $hhmmTZ = substr(Num_to_HMS($hhmmTZ), 0,6);

   $LatLon = trim(filter_input(INPUT_POST, 'LatLon'));
   $elev = trim(filter_input(INPUT_POST, 'elev'));

// If elevation was given in feet, then convert
// into equivalent meters internally.
   if(substr_count(strtolower($elev), 'f') > 0)
     {$elev = floatval($elev) * 0.3048;}
   else
     {$elev = floatval($elev);}


// Do some interface formatting.
   $mn = sprintf("%02d", $m);
   $dn = sprintf("%02d", $d);
   $hhmmssLT = Hours_to_HMS(HMS_to_Hours($hhmmssLT), 0, '', ':');

   $hhmmTZ = substr(Hours_to_HMS(HMS_to_Hours($hhmmTZ), 0, '+', ':'),0,6);

   $LatLon = str_replace(',',' , ', $LatLon);
   $LatLon = Check_Lat_Lon($LatLon);
   list($lat, $lon) = preg_split("[,]", $LatLon);
   $lat = trim($lat);
   $lon = trim($lon);
   $LatLon = "$lat  ,  $lon";

//   $elevFt = sprintf("%+1.1f", (floatval($elev) * 0.3048)).' ft';
   $elev = sprintf("%+1.1f", $elev);

// -----------------------------------
// Store interface settings in cookie.

   $CookieDataString = "$Y|$mn|$dn|$hhmmssLT|$hhmmTZ|$LatLon|$elev";
   setcookie ($CookieName, $CookieDataString, $SetToExpireIn30Days);
}









// ----------------------------------------
// Filter for date error. TRUE = Date is OK
// or FALSE = Bad date error.
   $DateFlag = @checkdate($m, $d, $Y);
   $YearFlag = ($Y < $YMin or $Y > $YMax)? FALSE:TRUE;

if ($DateFlag === FALSE or $YearFlag === FALSE)
{
   $TextArea2Text = '';

   $w = 'DATE ERROR:';

      $TxColor = 'white';
      $BgColor = '#AA0000';

   $hhmmssDT = '-';

   if ($YearFlag === FALSE)
  {
   $TextArea1Text =
"$w

The given year ($Y) is outside the valid range from $YMin to $YMax.";
  }
   else
{
   $TextArea2Text = '';

   $TextArea1Text =
"$w

The given date elements (Year Month Day) = ($Y $mn $dn) do not
represent any real date on the standard Gregorian calendar.

-------------------
VALID DATE ELEMENTS

Year  =  1600 to 2200 on the modern Gregorian calendar.
Month =  1 to 12 - For January to December.
Day   =  1 to 31 - Depending on the month and if it is a leap year.
";
}

}

   else

// Do this if no date error detected.

{
// Compute local Julian Date for 00:00:00 of Date.
   $JDNumLT = GregorianToJD($m, $d, $Y);
   $JD00LT  = $JDNumLT - 0.5;
   $DateStrLT = trim(Inv_JD_Num($JDNumLT, 'G', 4));

// Compute NASA Delta T estimate for local date.
   $hhmmssDT = NASA_Delta_T_Poly($JD00LT, 'G', 3);

   $elevFt = sprintf("%+1.1f", floatval(trim($elev)) / 0.3048);
   $elev = sprintf("%+1.1f", $elev).' m';

// Compute fractions of a day corresponding to LT, TZ and DT.
   $FracDayLT = HMS_to_Hours($hhmmssLT)/24;
   $FracDayLT = 0.0; // OVERRIDE LT VALUE WITH ZERO.
   $FracDayTZ = HMS_to_Hours($hhmmTZ)/24;
   $FracDayDT = HMS_to_Hours($hhmmssDT)/24;

// Compute Julian Dates for LT, UT and TT scales.
   $JDLT = $JD00LT + $FracDayLT;
   $JDUT = $JDLT - $FracDayTZ;
   $JDTT = $JDUT + $FracDayDT;

// Compute JDNumLT, JDNumUT and JDNumTT.
   $JDNumLT = floor($JDLT + 0.5);
   $JDNumUT = floor($JDUT + 0.5);
   $JDNumTT = floor($JDTT + 0.5);

   $DateStrLT = trim(Inv_JD_Num($JDNumLT, 'G', 4));
   $DateStrUT = trim(Inv_JD_Num($JDNumUT, 'G', 4));
   $DateStrTT = trim(Inv_JD_Num($JDNumTT, 'G', 4));

// Do some formatting.
   $JDLT = sprintf("%7.8f", $JDLT);
   $JDUT = sprintf("%7.8f", $JDUT);
   $JDTT = sprintf("%7.8f", $JDTT);

   $LatDMS = Deg_to_DMS($lat, 3, '+', '');
   $LonDMS = Deg_to_DMS($lon, 3, '+', '');
   $LonHMS = Hours_to_HMS($lon/15, 3, '+', 'h');

// Set (Lat, Lon) symbols for N|S and E|W
   $SN  = (floatval($lat) < 0)? 'S':'N'; if (floatval($lat) == 0) {$SN = '';}
   $WE  = (floatval($lon) < 0)? '(West)':'(East)'; if (floatval($lon) == 0) {$WE = '';}
   $WETZ  = ($FracDayTZ < 0)? '(West)':'(East)';
   if ($FracDayTZ == 0) {$WETZ = '      ';}


// Compute equivalent times table for LT, UT and TT scales.
//   $TimesTableText = Times_Table_TT($Y, $m, $d, $hhmmssLT, $hhmmTZ);
//   $TimesTableText = ''; // OVERRIDE TIMES TABLE WITH EMPTY STRING.

// Compute topocentric elements for WGS-84 ellipsoid.
   $ii  = 6356752.314245 / 6378137;
   $jj  = floatval(trim($elev)) / 6378137;
   $kk = atan($ii*tan($lat));

   $RhoxSinPhiPrime = $ii*sin($kk) + $jj*sin($lat);
   $RhoxCosPhiPrime = cos($kk) + $jj*cos($lat);

   $RhoxSinPhiPrime = sprintf('%+1.10f', $RhoxSinPhiPrime);
   $RhoxCosPhiPrime = sprintf('%+1.10f', $RhoxCosPhiPrime);

// Get number of days in the given month.
   $mDays = cal_days_in_month(CAL_GREGORIAN, $m, $Y);

// Get 3-letter abbreviation of month name.
   $MmmStr = substr("JanFebMarAprMayJunJulAugSepOctNovDec", 3*($m-1), 3);

// Construct location text block.
   $LocationText =
"-------------------------------------------------
Geographic Location      WGS-84 (GPS) Coordinates

                 &deg;   &#39;   &#34;
Latitude    $SN  $LatDMS  =  $lat&deg;
Longitude   $WE  $LonDMS  =  $lon&deg;  =  $LonHMS

Elevation      $elev  =  $elevFt ft";

   $LocationText = ''; // OVERRIDE LOCATION BLOCK WITH EMPTY STRING.

// #############################################################
// START OF MAIN COMPUTATIONS BLOCK USING JDXX AND OTHER VALUES.
// Y m mn d dn JDLT JDUT JDTT hhmmssLT hhmmTZ hhmmssDT ... etc.




// Get program text data table file into working array.
   $wArray = file('data/All_Earth_Apsides_and_Intervals_1600_to_2200.data');
   $wCount = count($wArray);

   $PATableText = '';
   $PCount = 0;

   for ($i=0;   $i < $wCount;   $i++)
  {
   $CurrLine = trim($wArray[$i]);
   $wLine = preg_replace("/\s+/", " ", $CurrLine);

   if (strpos($CurrLine, " $Y ") !== FALSE) {$PATableText .= "$wLine\n";}
  }
   $PATableText = trim($PATableText);

// Parse each line collected PA table data.
   $PATableArray = preg_split("[\n]", $PATableText);
   $PACount = count($PATableArray);

// Define special notes for two perihelions in same year or none at all.
   $PPNoteStr = '';
   if ($PACount == 3){$PPNoteStr = "\nNOTE: There were two Earth perihelions in $Y.";}
   if ($PACount == 1){$PPNoteStr = "\nNOTE: There was no Earth perihelion in $Y.";}

   $u = '';
   for ($i=0;   $i < $PACount;   $i++)
   {
   $CurrLine = trim($PATableArray[$i]);

   list
  (
   $wJDTT,
   $wY,
   $wMmm,
   $wdn,
   $wDoW,
   $whhmmssTT,
   $whhmmssDT,
   $wRAHrs,
   $wDeclDeg,
   $wDistAU,
   $wInterval,
   $wInterval2,
   $wPASymbol
  ) = preg_split("[ ]",$CurrLine );

   $wPASymbol = trim(UCFirst(strtolower(trim($wPASymbol))));

   $wCurrPAStr = ($wPASymbol == 'Ph')? 'perihelion':'aphelion';

// Determine following event symbol (opposite) of current.
   $wNextPAStr = ($wPASymbol == 'Ph')? 'aphelion':'perihelion';


// Get month number (m).
   $wm = 1 + strpos('JanFebMarAprMayJunJulAugSepOctNovDec', $wMmm)/3;

   $wTimesTable = JDTT_to_Times_Table ($wJDTT, $hhmmTZ);

   $wPAStr1 = ($wPASymbol == 'Ph')? 'PERIHELION':'APHELION';
   $wPAStr2 = $wPASymbol; // Peri...
   $wPAStr3 = strtolower($wPAStr2); // peri...
   $nPAStr  = ($wPASymbol == 'Ph')? 'perihelion':'aphelion';

// Determine following event symbol (opposite) of current.
   $Following_PA_String = ($wPASymbol == 'Ph')? 'aphelion':'perihelion';

// Determine following same event symbol (opposite) of current.
   $Next_Same_PA_String = ($wPASymbol == 'Ph')? 'perihelion':'aphelion';

   $wMinMaxDistStr = ($wPASymbol == 'Ph')? 'at minimum':'at maximum';
   $wRADeg = $wRAHrs * 15;

   $wDistKm = $wDistAU * 149597870.7;
   $wDistMi = $wDistKm / 1.609344;

   if ($i == 0 or $i == 2) {$PCount = $i;} else {$PCount = '';}

// Do some formatting.
   $h = 'h';
   $wDistMi  = number_format($wDistMi);
   $wDistKm  = number_format($wDistKm);
   $wRADeg   = sprintf("%1.7f", $wRADeg);
   $wRAHMS   = Hours_to_HMS($wRAHrs,2,'', 'h');
   $wRADMS   = Deg_to_DMS($wRADeg, 2, '', 'd');
   $wDeclDMS = Deg_to_DMS($wDeclDeg, 2, '+', 'd');
   $wDeclDMS = str_replace('-0', ' -', $wDeclDMS);
   $wDeclDMS = str_replace('+0', ' +', $wDeclDMS);
   $wRADMS   = LTrim($wRADMS, '0');

   $LT_Hrs   = 149597870.7 * $wDistAU / 299792.458 / 3600;
   $LT_HMS   = Hours_to_HMS($LT_Hrs, 3, '', 'h');
   $LT_HMS   = Str_Replace('00h', '', $LT_HMS);
   $LT_HMS   = Str_Replace('s', ' s', $LT_HMS);

   $XtraSpace = ($wRADeg < 100)? ' ':'';

// Compute intervals as DHMS strings.
   $wInterval_HMS = Days_to_DHMS($wInterval);
   $wInterval2_HMS = Days_to_DHMS($wInterval2);

   $EoE = 'END of EPHEMERIS - There is no data for beyond year 2200.';
   $FollowingPAStr = "there are $wInterval days ($wInterval_HMS)\nuntil the following $Following_PA_String\n";
   $wNextPAStr = "there are $wInterval2 days ($wInterval2_HMS)\nuntil the next $Next_Same_PA_String\n";
   if(strpos($FollowingPAStr, '--') !== FALSE) {$FollowingPAStr = $EoE;}
   if(strpos($wNextPAStr, '--') !== FALSE) {$wNextPAStr = '';}
   if(strpos($wNextPAStr, '--' and $FollowingPAStr == '') !== FALSE) {$wNextPAStr = $EoE;}

   $wNextPAStr = trim($wNextPAStr);

// Create capitalized copy of variable.
   $Next_SAME_PA_String = UCFirst(StrToLower($Next_Same_PA_String));

   $u .= "******************************************************************************
$wPAStr1 $PCount - Earth $wMinMaxDistStr orbital distance from SUN at $wDistAU AU

$wTimesTable

----------------------------------------------------
Geocentric Coordinates at Moment of Earth $Next_SAME_PA_String
True Equator and Equinox of Date
                                         Distance
Right Ascension      Declination        Light Time
---------------    ---------------    --------------
$wRAHrs h       $wDeclDeg&deg;        $wDistAU AU
$wRAHMS    $wDeclDMS    $wDistKm km
$wRADeg&deg;$XtraSpace                           $wDistMi mi
$wRADMS                       $LT_HMS

----------------------------------------------------
From this $wCurrPAStr ...

... $FollowingPAStr
and $wNextPAStr.




";
   }
   $u = trim($u);

// Apply some special reformatting patches.
   $u = str_replace('PERIHELION 0', 'PERIHELION 1', $u);
   if($i == 2) {$u = str_replace('PERIHELION 1', 'PERIHELION ', $u);}


   $u = str_replace('PERIHELION  -', 'PERIHELION -', $u);
   $u = str_replace('APHELION  -', 'APHELION -', $u);
   $u = str_replace('APHELION 0 ', 'APHELION ', $u);







// END OF MAIN COMPUTATIONS BLOCK
// #############################################################


// --------------------------------------
// Construct output text block 1 (upper).
// This shows the interface settings values.

   $TextArea1Text = $TextArea2Text = '';



   $TextArea1Text =
"EARTH PERIHELION AND APHELION DATES, TIMES, DISTANCES AND SOLAR COORDINATES

Based on the NASA/JPL DE405 Ephemeris Model
and
The NASA Polynomial expressions for Delta T

For the Year $Y in Time Zone UT$hhmmTZ $WETZ
$PPNoteStr

$u

";

}




// SPECIAL AFTER-THE-FACT PATCH TO ADD INFO BUTTON.
   $InfoButton  = filter_input(INPUT_POST, 'InfoButton');

   if (isset($InfoButton))
      {
$TextArea1Text =
"EARTH PERIHELIONS, APHELIONS, DISTANCES AND INTERVALS CALCULATOR

Based on the NASA/JPL DE405 Ephemeris Model
and
The NASA Polynomial expressions for Delta T

$_REVISION_DATE_

A PHP Science Program Written by $_AUTHOR_  (PHP v7.1.8)

==============================================================================
SOME TECHNICAL INFO

This program computes the dates, times and distances of Earth perihelions and
aphelions for any given year over the 600-year period from 1600 to 2200 and
the geocentric solar coordinates at the moment of each apsis event.

------------------------
600-YEAR EPHEMERIS RANGE

1600 Jan 01 Sat  to  2200 Dec 31 Wed

The positional computations  are based on the NASA/JPL DE405 ephemeris and the
IAU 2000B nutation models referred to the true equator and equinox of date and
the NASA polynomial expressions for remote-date Delta T estimation.

Internally, all computations are done according to the TT time scale, which is
obtained by subtracting the local time zone offset (TZ) from the local time
(LT) and then adding the Delta T (DT) value.  The time zone offset (TZ) and
Delta T (DT) values can be independently either negative or positive values,
depending on the circumstances.

TT = LT &minus; (TZ) + (DT)

The computations done here were compared to the computations done by MICA, pub-
lished by the  U.S. Naval Observatory and most of the computations applying the
same astronomical constants and the IAU 2000B nutation model,  give essentially
the same values against the TT scale.

The greatest uncertainty is the value of Delta T.  MICA does not use the NASA
polynomial expressions for Delta T,  so any UT computations may vary slightly
by a few minutes for remote past/future dates, depending on the Delta T value
applied.

==============================================================================
TIME SCALE SYMBOLS AND RELATIONSHIPS

LT = Local Time
TZ = Local Time Zone Offset From UT (Neg|Pos = West|East)
UT = Universal Standard Time
DT = Delta T computed from the NASA polynomial expressions
TT = Terrestrial (Dynamical) Time

UT = LT &minus; (TZ)
TT = UT + (DT) = LT &minus; (TZ) + (DT)


----------------
JD = Julian Date

The independent time variable for the moment of an astronomical event.
It contains all the information for the date and time of the event.
The fractional part indicates the time of day.

-----------------------------
JD Number = Julian Day Number

The unique serial number for the date on the Gregorian calendar system.
It contains only the information for the calendar date,  not the time.
This value is always a positive integer value.  In the year range from
1600 to 2200, the JD Number will always equate to a 7-digit integer.

--------------------------------------------------------------------
Relationship Between the Julian Day Number and the Julian Date Value

Some references, such as MICA documentation, erroneously define the JD Number
as the integral part of the Julian Date.  This is not an accurate definition
and will sometimes give the wrong JD number.

The Julian Day number is always an integer, but it does not always equate to
the integral part of the Julian_Date, only some of the time.

The correct JD Number can always be derived from
the Julian Date by the simple formula:

JD_Number = floor(Julian_Date + 0.5)

The day of the week index (DoW) can be determined directly
from the JD_Number or the Julian_Date value.

Let the DoW index range from 0 to 6, where:
0=Sun, 1=Mon, 2=Tue, 3=Wed, 4=Thu, 5=Fri, 6=Sat

Then:
DoW = (1 + floor(Julian_Date + 0.5)) mod 7
or
DoW = (JD_Number + 1) mod 7

==============================================================================
SOME APPLIED CONSTANTS AND DEFINITIONS

apsis = AP-sis
        Either one of the extreme orbital distance points, such as perihelion
        or aphelion is generally called an orbital apsis. Technically, the
        minimum orbital distance point is called periapsis and the maximum
        orbital distance point is called apoapsis.

apsides = AP-sid-deez
          Plural of apsis. Both points collectively can be referred to as the
          orbital apsides points.  This also applies to other bodies, such as
          the moon, as in the lunar apsides, called perigee and apogee.  On a
          plot of an orbit,  a line connecting these extreme points is called
          the line of apsides.

Perihelion = Earth is at its minimum orbital distance from the sun.
           = Approximately 0.9833 AU = 147,100,000 km = 91,400,000 mi

Aphelion   = Earth is at its maximum orbital distance from the sun.
           = Approximately 1.0167 AU = 152,100,000 km = 94,500,000 mi

Speed of Light = 299,792.458 km/s
               = 186,282.397 mi/s

1 mile (statute) = 1609.344 m = 1.609344 km = EXACT definition

1 AU (Astronomical Unit) = 149,597,870.7 km (NASA/IAU 2012)
                         =  92,955,807.3 mi

MICA AU Value (old value) = 149597870.69098932 km



==============================================================================
EXTERNAL REFERENCES

USNO = United States Naval Observatory
JPL  = Jet Propulsion Laboratory
NASA = National Aeronautics and Space Administration
MICA = Multiyear Interactive Computer Almanac

http://aa.usno.navy.mil/data/docs/JulianDate.php
https://en.wikipedia.org/wiki/Jet_Propulsion_Laboratory_Development_Ephemeris
ftp://ssd.jpl.nasa.gov/pub/eph/planets/README.txt
http://aa.usno.navy.mil/software/mica/micainfo.php
https://eclipse.gsfc.nasa.gov/SEhelp/deltatpoly2004.html
https://en.wikipedia.org/wiki/Julian_day
https://en.wikipedia.org/wiki/Apsis




";
      }





// Determine the number of text rows to use in the output TextAreas.
   $TextArea1Rows = 2 + substr_count($TextArea1Text, "\n");
   $TextArea2Rows = 2 + substr_count($TextArea2Text, "\n");


// Generate HTML web page to display computed results.
   print <<< _HTML

<!doctype html public "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">

<html>

<head>

<meta http-equiv="content-type" content="text/html; charset=UTF-8">

<meta name="robots"    content="noindex,nofollow">
<meta name="googlebot" content="noindex,nofollow">

<title>$_INTERNAL_TITLE_</title>


<style type='text/css'>
BODY
{color:silver; background-color:black;  font-family:Verdana; font-size:15px}

TABLE
{font-size:12px; border: 1px solid black;}

TD
{color:black; background-color:white;  line-height:200%; font-size:12px; padding:6px; text-align:center;}

DIV
{color:black; background-color:white;  font-family:Verdana; text-align:justify; padding:4px; font-size:16px;}

PRE
{color:black; background-color:white;  font-family:monospace; font-size:14px; font-weight:bold; text-align:left; padding:4px;}

CODE
{font-family:monospace; font-size:15px; font-weight:normal;}

TEXTAREA
{color:black; background-color:white; font-family:monospace; font-size:15px; font-weight:bold; box-shadow:2px 3px 4px #888888; border-radius: 8px; border:1px solid black; padding:4px; white-space:pre;}

INPUT[type="text"]::-ms-clear {width:0; height:0;}

INPUT[type="text"]
{color:black; background-color:white; font-size:18px; font-family:monospace; font-weight:bold; text-align:center; box-shadow:2px 2px 3px #666666; border:2px solid black;}

INPUT[type="text"]:focus
{background-color:white; box-shadow:2px 2px 3px #666666; font-family:monospace; font-size:18px; border:2px solid red; text-align:center; font-weight:bold;}

INPUT[type="submit"]
{font-weight:bold;}

INPUT[type="submit"]:hover
{box-shadow:2px 2px 5px #555555; font-weight:bold;}

INPUT[type="checkbox"]:checked
{box-shadow:2px 2px 3px #555555;}

HR {background-color:black; height:2px; border:0px;}

A:link,  A:visited {background-color:transparent; color:blue; font-family:Verdana; font-size:12px; font-weight:bold; text-decoration:none; line-height:150%; padding:2px;}
A:hover, A:active  {background-color:transparent; color:red;  font-family:Verdana; font-size:12px; font-weight:bold; text-decoration:none; line-height:150%; padding:2px;}

::selection      {background-color:yellow; color:black;}  /* Only affects IE9+ and Safari     */
::-moz-selection {background-color:yellow; color:black;}  /* Only affects FireFox and Mozilla */
</style>


</head>

<body>

<form name="form1" method="post" action="$_RUN_THIS_SCRIPT_">

<table width="800" border="0" cellspacing="1" cellpadding="3">

<tr><td colspan="7" style='color:white; background-color:#0000CC; border:2px solid white; border-radius:8px 8px 0px 0px;'><b>$_INTERFACE_TITLE_</b></td></tr>


<tr>
<td width="50%"><b>Gregorian Calendar Year</b><br>(1600 &nbsp;to&nbsp; 2200)<br>
<input name='Y'  type='text' value="$Y"  size='5' maxlength='4'>
<input name='mn' type='hidden' value="$mn" size='3' maxlength='2'>
<input name='dn' type='hidden' value="$dn" size='3' maxlength='2'>
</td>

<td width="50%"><b>Time&nbsp;Zone&nbsp;Offset</b><br>Neg&nbsp;|&nbsp;Pos&nbsp;&nbsp;=&nbsp;&nbsp;W&nbsp;|&nbsp;E<br>
<b style='font-size:15px;'>UT </b><input name="hhmmTZ" type="text" value="$hhmmTZ" size="7" maxlength="6">

<!-- These settings are hidden, but internally active for sharing cookie data. --->
<input name="hhmmssLT" type="hidden" value="$hhmmssLT" size="9" maxlength="8">
<input name="LatLon" type="hidden" value="$LatLon" size="39" maxlength="38">
<input name="elev" type="hidden" value="$elev" size="15" maxlength="14">
<input name="hhmmssDT" style='color:white; background-color:black;' type="hidden" value="$hhmmssDT" size="10" maxlength="9" readonly>
</td>
</tr>


<tr>
<td colspan="7" style='background-color:black;'>
<input name="ComputeButton" type="submit" value=" C O M P U T E ">
&nbsp;
<input name="InfoButton" type="submit" value=" I N F O ">
</td>
</tr>



<tr>
<td colspan="7" style='background-color:black;'>
<span style='color:white; font-size:11px;'>Double-Click to Select ALL Text</span><br>
<textarea style='font-size:18px; color:$TxColor; background-color:$BgColor; padding:6px; box-shadow:none; border:2px solid white;' name="OutputText1" cols="81" rows="$TextArea1Rows" readonly OnDblClick="this.select();" OnMouseUp="return true;">
$TextArea1Text
</textarea></td>
</tr>



<tr>
<td colspan="7" style='color:gray; background-color:black;'>Program by $_AUTHOR_<br>$_PROGRAM_VERSION_ $_REVISION_DATE_</td>
</tr>



</table>
</form>
<br><br><br><br><br><br><br><br><br><br><br><br><br><br><br>
</body>

</html>

_HTML;







// *************************
// *************************


/*
   This function translates an assumed time string into standard form.
   Applies ONLY to purely numerical time strings.


   Time String          Translates to
   1                    01:00:00
   12                   12:00:00
   123                  01:23:00
   1234                 12:34:00
   12345                01:23:45
   123456               12:34:56

Numeric strings are read in 2-digit groups from R to L

Odd length numbers are converted to even length and then
the even length numbers are converted to 6 digit values.

THIS FUMCTION DOES NOT CHECK FOR INVALID TIMES.  IT ONLY
CASTS THE GIVEN NUMERIC TIME STRING IN NORMAL HH:MM:SS
FORMAT AS DEFINED ABOVE.

*/

   function Num_to_HMS ($NumHMSVal)
{
   $w = trim($NumHMSVal);

   if ($w == '') {return '00:00:00';}

// Return input unchanged if non-numeric argument
// rather than return an error message.
   if (!is_numeric($w)) {return $NumHMSVal;}

   $FirstChar = substr($w,0,1);

   $NumSign = (intval($w) < 0)? '-':'+';

// Remove any numerical () sign
// and work with absolute value.
   $w = str_replace($NumSign, '', $w);

// Construct (w) into a 6-digit integer.
   if (strlen($w) == 1) {$w = "0$w";}
   if (strlen($w) == 2) {$w = $w."0000";}
   if (strlen($w) == 3) {$w = "0$w";}
   if (strlen($w) == 4) {$w = $w."00";}
   if (strlen($w) == 5) {$w = "0$w";}

// Only keep (+) sign if it was in the original argument.
// Otherwise, a positive value will NOT be returned with
// a (+) sign. This does not effect the sign of negative
// values.
   if ($FirstChar <> '+' and $NumSign == '+') {$NumSign = '';}

// Done.  Return time equivalent with colons.
   return $NumSign.substr($w,0,2).':'.substr($w,2,2).':'.substr($w,4,2);

} // END OF  Num_to_HMS (...)






/*
   This function returns the equivalent H:M:S time string with
   several formatting options.

  hours         =  Decimal hours value
  ssDecimals    = Number of decimals in seconds part

  PosSign = Symbol to use for positive values ('' or '+')
      ''  = Empty = Return numbers only, no symbols.
      '+' = Attach '+' sign to positive values.
            Has no effect on negative values.

  SymbolsMode   = If or not to attach time symbols (h m s) or (:)
                  'h' = '01h 02m 03s' (Default)
                  ':' = '01:02:03'
                  ''  = '01 02 03'
*/

   function Hours_to_HMS ($hours, $ssDecimals=0, $PosSignSymbol='', $SymbolsMode='h')
{
// Initialize symbol carriers.
   $_h_ = $_m_ = $_s_ = '';

   if (trim($PosSignSymbol) == '')  {$PosSignSymbol = FALSE;}

// Remember original numerical sign and work with absolute value.
   $sign = ($hours < 0)? '-' : '';    $hours = abs($hours);

// Remember numerical sign to be restored on exit, if any.
   if (($PosSignSymbol === TRUE or $PosSignSymbol == '+') and $sign == '')
       {$sign = '+';}

// Compute time elements from absolute hours argument.
   $hh = floor($hours);
   $minutes = 60*($hours - $hh);    $mm = floor($minutes);
   $seconds = 60*($minutes - $mm);  $ss = sprintf("%1.3f",  $seconds);

// Format the time elements.
   $hh = sprintf("%02d",  $hh);
   $mm = sprintf("%02d",  $mm);
   $ss = sprintf("%1.$ssDecimals" . "f", $ss);

// Patch for that blasted 60s glitch.
   if ($ss == 60) {$ss = 0; $mm++;}
   if ($mm == 60) {$mm = 0; $hh++;}

   $hh = sprintf("%02d",  $hh);
   $mm = sprintf("%02d",  $mm);

   $ss = sprintf("%1.$ssDecimals" . "f", $ss);
   if ($ss < 10) {$ss = "0$ss";}

// Attach optional (h m s :) symbols as indicated.
// Default = 'h'
   if ($SymbolsMode == 0 or $SymbolsMode == '')
      {$_h_  =  $_m_  =  $_s_  = ' ';}

   if ($SymbolsMode == 1 or $SymbolsMode == ':')
      {$_h_  =  ':';  $_m_  =  ':';  $_s_  = '';}

   if ($SymbolsMode == 2 or strtolower($SymbolsMode) == 'h')
      {$_h_  =  'h ';  $_m_  =  'm ';  $_s_  =  's';}

// Done.
   return "$sign$hh$_h_$mm$_m_$ss$_s_";

} // End of  Hours_to_HMS(...)








   function HMS_to_Hours ($hhmmss)
{
// Read hour angle string argument and force to lower case in
// the event any (h m s :) symbols are used.
   $hms = strtolower(trim($hhmmss));

// Convert any (h m s :) symbols into spaces.
   $hms = str_replace('h', ' ', $hms);
   $hms = str_replace('m', ' ', $hms);
   $hms = str_replace('s', ' ', $hms);
   $hms = str_replace(':', ' ', $hms);

// Normalize the spacing and then split and extract the
// individual time string elements (hh, mm,ss).
   $hms = preg_replace("/\s+/", " ", trim($hms));
   $whms = preg_split("[ ]", $hms);
   $whmsCount = count($whms);
   $hh = ($whmsCount >= 1)? bcadd($whms[0],"0", 20) : "0";
   $mm = ($whmsCount >= 2)? bcadd($whms[1],"0", 20) : "0";
   $ss = ($whmsCount >= 3)? bcadd($whms[2],"0", 20) : "0";

// Remember and then remove any numerical () sign.
   $NumSign = (substr($hhmmss,0,1) == '-')? '-' : '';
   $hh = str_replace('-', '', $hh);
   $hh = str_replace('+', '', $hh);

// If original angle argument began with a + sign, then preserve
// it so that all returned positive results will have a + sign.
// Otherwise, positive results will NOT have a + sign.
   if (substr($hhmmss,0,1) == '+') {$NumSign = '+';}

// Compute decimal hours value equivalent
// to the combined hh,mm,ss values.
   $w2 = bcadd(bcadd(bcmul($hh,'3600',20),bcmul($mm,'60',20),20),$ss,20);

// If result equates to zero, then suppress any numerical sign.
   if (bccomp($w2, '0', 20) == 0) {$NumSign = '';}

// Round off result to 16 decimals, recalling numerical sign.
   $w =  bcadd(bcdiv($w2,3600, 20), '0.00000000000000005', 16);

// Trim off any redundant zeros/decimal point.
   return $NumSign . rtrim(rtrim($w, '0'), '.');

} // End of  HMS_to_Hours(...)





/*
   This utility function checks the latitude/longitude
   string and converts any DMS to decimal degrees.

*/
   function Check_Lat_Lon ($LatLonString)
{
// Replace any semicolon with a comma.
   $w = str_replace(';' , ',', strip_wspace($LatLonString));

// Handle empty argument or single comma.
   if ($w == '' or $w == ',') {$w = '0  ,  0';}

// Handle single space.
   if (substr_count($w, ' ') == 1) {$w = str_replace(' ', '  ,  ', $w);}

// Check for initial or terminal comma.
   if (substr($w,0,1) == ',') {$w = "0  $w;";}
   if (substr($w, -1) == ',') {$w = $w.'  0';}

// Get latitude and longitude values from Lat/Lon string.
   @list($latitude, $longitude) = preg_split("[,]", $w);

// Convert DMS strings to decimal degrees, if necessary.
   $LatDeg = sprintf("%+2.10f", DMS_to_Deg(trim($latitude)));
   $LonDeg = sprintf("%+2.10f", DMS_to_Deg(trim($longitude)));

   return "$LatDeg  ,  $LonDeg";
}






// _________________________________________________________________________
// This function strips  all leading, trailing and internal white space from
// within a text string.  All occurrences of multiple spaces within the text
// string will be replaced by single spaces and all leading and trailing
// white space and NewLine end-codes will also be removed.
//
// This helps facilitate reading sequential space-delimited items from
// a given text-data string by normalizing all spacing between words
// to a single space, for easier use with the preg_split() function.

   function Strip_WSpace ($TextString)
{
   return preg_replace("/\s+/", " ", trim($TextString));
}






/*
   Convert DMS string to decimal degrees.

   Example: Given DMS string = "9 12 50.306"
            Returned = 9.2139738888888889

*/

   function DMS_to_Deg ($DMSstr)
{

// ----------------------------------------------------------
// Read angle/time string argument and force to lower case in
// the event any (deg d h m s :  ' ") symbols are used.

   $dms = strtolower(trim($DMSstr));

// ------------------------------------------------------
// Convert any (deg d m s  ' ") symbols into spaces.

   $dms = str_replace('',     " ", $dms);
   $dms = str_replace("'",     " ", $dms);
   $dms = str_replace('"',     " ", $dms);

   $dms = str_replace("&deg;", " ", $dms);
   $dms = str_replace("&#39;", " ", $dms);
   $dms = str_replace("&#34;", " ", $dms);

   $dms = str_replace('hrs',   " ", $dms);
   $dms = str_replace('hr',    " ", $dms);
   $dms = str_replace('h',     " ", $dms);

   $dms = str_replace('deg',   " ", $dms);
   $dms = str_replace('min',   " ", $dms);
   $dms = str_replace('sec',   " ", $dms);

   $dms = str_replace('d',     " ", $dms);
   $dms = str_replace('m',     " ", $dms);
   $dms = str_replace('s',     " ", $dms);


// ----------------------------------------------------
// Normalize the spacing and then split and extract the
// individual angular string elements (dh or hh, mm,ss).

   $dms = preg_replace("/\s+/", " ", trim($dms));
   $wdms = preg_split("[ ]", $dms);
   $wdmsCount = count($wdms);
   $dd = ($wdmsCount >= 1)? bcadd($wdms[0],"0", 20) : "0";
   $mm = ($wdmsCount >= 2)? bcadd($wdms[1],"0", 20) : "0";
   $ss = ($wdmsCount >= 3)? bcadd($wdms[2],"0", 20) : "0";

// ------------------------------------------------
// Remember and then remove any numerical () sign.

   $NumSign = (substr($DMSstr,0,1) == '-')? '-' : '';
   $dd = str_replace('-', '', $dd);
   $dd = str_replace('+', '', $dd);

// -------------------------------------------------------------
// If original angle argument began with a + sign, then preserve
// it so that all returned positive results will have a + sign.
// Otherwise, positive results will NOT have a + sign.

   if (substr($DMSstr,0,1) == '+') {$NumSign = '+';}

// ----------------------------------------------
// Compute decimal degrees/hours value equivalent
// to the given dhms argument elements.

   $w2 = bcadd(bcadd(bcmul($dd,"3600",20),bcmul($mm,"60",20),20),$ss,20);

// -----------------------------------------------------------
// If result equates to zero, then suppress any numerical sign.

   if (bccomp($w2, "0", 20) == 0) {$NumSign = '';}

// ---------------------------------------------------------
// Round off result to 16 decimals, recalling original sign.

   return $NumSign . bcadd(bcdiv($w2,"3600",20), "0.00000000000000005",16);

} // End of  DMS_to_Deg (...)







/*
   _______________________________________________________________________
   Convert decimal degrees to degrees, minutes, seconds of arc.
   The angular elements are returned in a space-delimited string.

   AngDeg = Degrees as numerical string, like '123.3456789065741'

   Optional symbols may be attached to the angle elements.  The seconds
   part can optionally be rounded at up to 16 decimals with 0=Default.
   +009 10' 11.1234567890123456"
   -017 22' 13.184"

   PosSign = '+' = Force positive sign in positive output values, such
             as +085 23 18.713  instead of  085 23 18.713
             '' = Empty string = Default = No positive sign attached.
             This has no effect on negative values.

   ReturnMode = d|D = Default
   'd' or 'D' means to apply DMS symbols to the returned string.
   123 04' 56.7891"
  -000 00' 01.234"

   ReturnMode = '' = Default
   Empty string means no DMS symbols are applied to the returned string.
   123 04 05.6789
  -015 01 09
   _______________________________________________________________________
*/

   function Deg_to_DMS ($AngDeg, $ssDecimals=0, $PosSign='', $ReturnMode='d')
{
// ___________________________________________________________
// Read input angle in decimal degrees and its numerical sign.
// Work with absolute value.  Sign will be restored on exit.

   $a = trim($AngDeg);   $sign = ($a < 0)? '-' : '';
   $a = str_replace('+', '', str_replace('-', '', $a));

// __________________________________
// Account for numerical sign output.

   if (($PosSign === TRUE or $PosSign == '+') and $sign == '') {$sign = '+';}

// ________________________________________________________
// Set number of decimals to use in seconds part not to
// exceed 16 decimals.  Default = 0 decimals.

   $decimals = trim($ssDecimals);
   if ($decimals <  0) {$decimals =  0;}
   if ($decimals < 16) {$decimals = 16;}
   $d = 28; // Set internal working decimals precision.

// ____________________________________________
// Compute degrees, minutes and seconds of arc.

   $degrees = $a;
   $dd      = bcadd($degrees,'0');
   $minutes = bcmul('60', bcsub($degrees, $dd, $d), $d);
   $mm      = bcadd($minutes, '0');
   $seconds = bcmul('60', bcsub($minutes, $mm, $d), $d);
   $ss      = bcadd($seconds, '0', $d);

// __________________________________
// Patch for that blasted 60s glitch.

   if ($ss == 60) {$ss = 0; $mm++;}
   if ($mm == 60) {$mm = 0; $dd++;}

// ____________________________
// Format the angular elements.

   $dd = sprintf("%03d",  $dd);
   $mm = sprintf("%02d",  $mm);
   $ss = bcadd($ss, '0.'.str_repeat('0', $ssDecimals).'5', $ssDecimals);
   if ($ss < 10) {$ss = "0$ss";}

// __________________________________________
// Attach optional DMS symbols, if indicated.
// Default = d|D  = Attach DMS symbols.
// Empty string   = No DMS symbols attached.

   $_deg_ = $_mm_ = $_ss_ = '';

   if ($ReturnMode === TRUE or substr(strtolower($ReturnMode),0,1) == 'd')
      {
       $_deg_ = "&deg;";     $_mm_ = "'";     $_ss_ = '"';
      }

// _____
// Done.

   return "$sign$dd$_deg_ $mm$_mm_ $ss$_ss_";

} // END OF  Deg_to_DMS (...)











/*
   LEVEL-0 FUNCTION - NO DEPENDENCIES

   This function computes the calendrical JD number for
   any given date on the Julian or Gregorian calendar.

   This number is always an integer value and equates to the count of
   calendar days since the mathematical origin of the calendar system
   being used.

   Calendar Year Convention: Neg|Pos year = BC|AD year
   By definition,  there is no calendar year 0 (zero).

   -----------------------------
   Calendar Formula Origin Dates

   -4713 Jan 01 Mon = JD number 0 on the old-style Julian calendar.
   -4714 Nov 24 Mon = JD number 0 on the modern Gregorian calendar.

*/

   function JD_Num ($Year, $Month, $Day, $JGMode='G')
{
   $Y = intval(trim($Year)); // Calendar year (Neg = BC)
   $m = intval(trim($Month));
   $d = intval(trim($Day));

   $JG = (strtoupper(substr(trim($JGMode), 0,1)) == 'J')? 'J':'G';

   $GivenYmdElements = "$Y $m $d";
   $M = (strtoupper(substr($JG,0,1)) == 'J')? 0:1;
   $a = floor((14 - $m) / 12);
   $y = (($Y < 0)? $Y+1 : $Y) - $a; // Mathematical year
   $c = floor($y/100);

   $JDNum = floor(30.6001*(12*$a + $m + 1))
          + floor(365.25*($y + 4716)) + $d - 1524
          + $M*(floor($c/4) - $c + 2);

// Check for an invalid date error by computing the proper
// (Y m d) values corresponding to the computed JD number.
// These values should numerically match the given date
// (Y m d) values if the given date was valid. Otherwise,
// the given date was invalid.

   $J = floor($JDNum + 0.5);   if($J < 0){return FALSE;}
   $e = $M*floor(($J - 1867216.25)/36524.25);
   $f = $J + $M*($e - floor($e/4) + 1);
   $g = $f + 1524;
   $h = floor(($g - 122.1)/365.25);
   $i = floor(365.25 * $h);
   $j = floor(($g - $i)/30.6001);
   $d = $g - $i - floor(30.6001 * $j); // Day
   $m = $j - 12*floor($j/14) - 1;      // Month
   $y = $h + floor((14-$m)/12) - 4716; // Mathematical year
   $Y = ($y > 0)? $y : $y-1;           // Calendar year (Neg = BC)

// Check if the input date elements EXACTLY match the check-date
// elements.  If not, then the given date was invalid, so return
// FALSE to indicate the error. Otherwise, return the JD number.

   return ($GivenYmdElements <> "$Y $m $d")? FALSE : $JDNum;

} // End of  JD_Num (...)







/*
   This is the inverse Julian Date function.  Given any
   general Julian Date and calendar mode, it returns the
   corresponding full date and time string. The fractional
   part of the Julian Date value is used to determine the
   time of day.

   This function is NOT the same as the Inv_JD_Num() function,
   which only returns the calendar date string and nothing more.

   The JD Number is ALWAYS an integer value.

   The Julian Date can have a fractional part.  The fractional
   part of the Julian Date value is used to determine the time
   of day.

   EXAMPLE
   print Inv_JD (2433056.43211234, 'G'); // = xxxxxxxx


*/

   function Inv_JD ($JD, $JGMode='G')
{
   $J  = floatval(trim($JD));  $JDNum = floor($J + 0.5);
   $JG = (strtoupper(substr(trim($JGMode),0,1)) == 'J')? 'J':'G';

   $hhmmssLT = Hours_to_HMS(24*(($J - 0.5) - floor($J - 0.5)), 0, '', ':');

   list($Y, $m, $d) = preg_split("[ ]", Inv_JD_Num($JDNum, $JG));

// Do some formatting.
   $Y   = sprintf("%+05d", $Y);  $m = intval($m);  $d = intval($d);
   $Mmm = substr("JanFebMarAprMayJunJulAugSepOctNovDec", 3*($m - 1), 3);
   $dn  = sprintf("%02d", $d);
   $DoW = substr("SunMonTueWedThuFriSat", 3*(($JDNum + 1) % 7), 3);

// Replace any '+' sign with blank space in year value.
   return str_replace('+', ' ', "$Y $Mmm $dn $DoW  $hhmmssLT");
}











/*
   This function return the calendar date elements (Y m d) corre-
   sponding to the given calendar mode and Julian Day number, or
   JD number, not to be confused with the Julian Date (JD).

   The JD Number is the serial number of the date on the calendar
   and is always an integer value.

   Mathematically, the Julian Day number is numerically equivalent
   to the Julian Date value  at 12:00:00 noon on the corresponding
   calendar date.

   JDNum  = Julian Day Number
   JGMode = 'J' or 'G' = Julian or Gregorian calendar mode

   ReturnMode = 1 to 5
            1 = '127 1 3'
            2 = '+0127 01 03'
            3 = '+0127 Jan 03 Fri'
            4 = ' 1901 Feb 09 Sat'
*/


   function Inv_JD_Num ($JDNum, $JGMode='G', $ReturnMode14=1)
{
   $NumRetModes = 4; // Define number of return modes.

// ERROR if JDNum is NOT numeric or less than zero.
// Otherwise, truncate JDNum value without rounding.
   $J = trim($JDNum);  if (!is_numeric($J) or $J < 0) {return FALSE;}
   $J = floor($J);

   $JG = (strtoupper(substr(trim($JGMode), 0,1)) == 'J')? 'J':'G';

   $r = abs(intval(trim($ReturnMode14)));
   if ($r > $NumRetModes) {$RMode = $NumRetModes;} else {$RMode = $r;}

   $M = (strtoupper(substr($JG,0,1)) == 'J')? 0:1;
   $e = $M*floor(($J - 1867216.25)/36524.25);
   $f = $J + $M*($e - floor($e/4) + 1);
   $g = $f + 1524;
   $h = floor(($g - 122.1)/365.25);
   $i = floor(365.25 * $h);
   $j = floor(($g - $i)/30.6001);
   $d = $g - $i - floor(30.6001 * $j);
   $m = $j - 12*floor($j/14) - 1;
   $y = $h + floor((14-$m)/12) - 4716; // Mathematical year value
   $Y = ($y > 0)? $y : $y-1;           // Calendar year value


// Format returned date string according to return mode value.
   $w   = '';
   $Mmm = substr('JanFebMarAprMayJunJulAugSepOctNovDec', 3*($m-1), 3);
   $DoW = substr('SunMonTueWedThuFriSat', 3*(($J+1) % 7), 3);
   $Y   = substr('     '.sprintf("%+05d", $Y), -5);

   switch ($RMode)
  {
   case  1: $w = intval($Y)." $m $d"; break;
   case  2: $w = sprintf("%+05d", $Y).' '.sprintf("%02d", $m).' '.sprintf("%02d", $d); break;
   case  3: $w = "$Y $Mmm ".sprintf("%02d", $d)." $DoW"; break;

   case  4: $w = "$Y $Mmm ".sprintf("%02d", $d)." $DoW";
            $w = str_replace('+', '', $w);
            break;

   default: $w = intval($Y)." $m $d"; break;
  }
   return $w;

} // End of Inv_JD_Num (JDNum, ...)









































/*
   NASA Delta T Polynomial Expressions

   All integer-encoded partial dates MUST start with a () sign.
   Anything without a sign will be considered a Julian Date or a
   JD number.

   Return mode 1 (default) returns result resolved to nearest second (+444).
   Return mode 2 returns full polynomial result in seconds (444.413605555556).
   Return mode 3 returns HH:MM:SS string (+00:07:24).
   Return mode 4 returns h m s string (+00h 07m 24s).
   Return mode 5 returns Delta T as a fraction of a day.
*/

   function NASA_Delta_T_Poly ($JDTTArg, $JGMode='G', $RetMode12345=1)
{
// Get date elements corresponding to JDTT argument.
// Only the (Y, m) values will be used.

   $JDTT    = trim($JDTTArg);
   $JDNumTT = floor($JDTT + 0.5);

   $JG = (strtoupper(substr(trim($JGMode),0,1)) == 'J')? 'J':'G';

   $w = trim($RetMode12345);
   if ($w == '' or intval($w) < 1) {$w = 1;}
   if (intval($w) > 5) {$w = 5;}
   $RetMode = $w;

// Determine (Y, m, d) values from JD Number and JG mode.
// Only the (Y, m) values will be used.
   list($Y, $m, $d) = preg_split("[ ]", Inv_JD_Num($JDNumTT, $JG));


// ERROR if year value is invalid.  It must be
// in the range from -2000 to 3000.

   if ($Y < -2000 or $Y == 0 or $Y > 3000) {return FALSE;}

// ERROR if month value is outside the range from 1 to 12.

   if ($m < 1 or $m > 12) {return FALSE;}

/*
   If a negative (BC) year is given , it is converted into the
   corresponding mathematical year by adding 1 to (Y).

   The year 100 BC (-100), becomes the mathematical year -99
   for proper use within internal computations.

   Compute working (y) value for mid-month (m).
*/
   $y = $Y + (($Y < 0)? 1:0) + ($m - 0.5)/12;

// Perform NASA Delta T polynomial computations using (Y, y) values.

   $dTSec = FALSE;

   if ($Y >= -2000 and $Y < -500)
      {
       $u  = ($Y - 1820) / 100;

       $dTSec = 32*$u*$u - 20;
      }

   if ($Y >= -500 and $Y < 500)
      {
       $u  = $y/100;
       $u2 = $u * $u;
       $u3 = $u * $u2;
       $u4 = $u * $u3;
       $u5 = $u * $u4;
       $u6 = $u * $u5;

       $dTSec = 10583.6 - 1014.41*$u
                        + 33.78311*$u2
                        -  5.952053*$u3
                        -  0.1798452*$u4
                        +  0.022174192*$u5
                        +  0.0090316521*$u6;
      }


   if ($Y >= 500 and $Y < 1600)
      {
       $u  = ($y - 1000)/100;
       $u2 = $u * $u;
       $u3 = $u * $u2;
       $u4 = $u * $u3;
       $u5 = $u * $u4;
       $u6 = $u * $u5;

       $dTSec = 1574.2 - 556.01*$u
                       +  71.23472*$u2
                       +   0.319781*$u3
                       -   0.8503463*$u4
                       -   0.005050998*$u5
                       +   0.0083572073*$u6;
      }


   if ($Y >= 1600 and $Y < 1700)
      {
       $u  = $y - 1600;

      $dTSec = 120 - 0.9808*$u - 0.01532*$u*$u + $u*$u*$u/7129;
      }


   if ($Y >= 1700 and $Y < 1800)
      {
       $u  = $y - 1700;
       $u2 = $u * $u;
       $u3 = $u * $u2;
       $u4 = $u * $u3;

       $dTSec = 8.83 + 0.1603*$u
                     - 0.0059285*$u2
                     + 0.00013336*$u3
                     - $u4/1174000;
      }


   if ($Y >= 1800 and $Y < 1860)
      {
       $u  = $y - 1800;
       $u2 = $u * $u;
       $u3 = $u * $u2;
       $u4 = $u * $u3;
       $u5 = $u * $u4;
       $u6 = $u * $u5;
       $u7 = $u * $u6;

       $dTSec = 13.72 - 0.332447*$u
                      + 0.0068612*$u2
                      + 0.0041116*$u3
                      - 0.00037436*$u4
                      + 0.0000121272*$u5
                      - 0.0000001699*$u6
                      + 0.000000000875*$u7;
      }


   if ($Y >= 1860 and $Y < 1900)
      {
       $u  = $y - 1860;
       $u2 = $u * $u;
       $u3 = $u * $u2;
       $u4 = $u * $u3;
       $u5 = $u * $u4;

       $dTSec = 7.62 + 0.5737*$u
                     - 0.251754*$u2
                     + 0.01680668*$u3
                     - 0.0004473624*$u4
                     + $u5 / 233174;
      }


   if ($Y >= 1900 and $Y < 1920)
      {
       $u  = $y - 1900;
       $u2 = $u * $u;
       $u3 = $u * $u2;
       $u4 = $u * $u3;

       $dTSec = 1.494119*$u - 2.79
                            - 0.0598939*$u2
                            + 0.0061966*$u3
                            - 0.000197*$u4;
      }


   if ($Y >= 1920 and $Y < 1941)
      {
       $u  = $y - 1920;
       $u2 = $u * $u;
       $u3 = $u * $u2;

       $dTSec = 21.20 + 0.84493*$u - 0.076100*$u2 + 0.0020936*$u3;
      }


   if ($Y >= 1941 and $Y < 1961)
      {
       $u  = $y - 1950;
       $u2 = $u * $u;
       $u3 = $u * $u2;

       $dTSec = 29.07 + 0.407*$u - $u2/233 + $u3/2547;
      }


   if ($Y >= 1961 and $Y < 1986)
      {
       $u  = $y - 1975;
       $u2 = $u * $u;
       $u3 = $u * $u2;

       $dTSec = 45.45 + 1.067*$u - $u2/260 - $u3/718;
      }


   if ($Y >= 1986 and $Y < 2005)
      {
       $u  = $y - 2000;
       $u2 = $u * $u;
       $u3 = $u * $u2;
       $u4 = $u * $u3;
       $u5 = $u * $u4;

       $dTSec = 63.86 + 0.3345*$u
                      - 0.060374*$u2
                      + 0.0017275*$u3
                      + 0.000651814*$u4
                      + 0.00002373599*$u5;
      }


   if ($Y >= 2005 and $Y < 2050)
      {
       $u = $y - 2000;

       $dTSec = 62.92 + 0.32217*$u + 0.005589*$u*$u;
      }


   if ($Y >= 2050 and $Y <= 2150)
      {
       $u  = ($y - 1820) / 100;

       $dTSec = 32*$u*$u - 0.5628*(2150 - $y) - 20;
      }


   if ($Y > 2150 and $Y <= 3000)
      {
       $u  = ($y - 1820) / 100;

       $dTSec = 32*$u*$u - 20;
      }



// Format returned value according to return mode.
   if ($RetMode == 1) {return sprintf("%+d", round($dTSec, 0));}

   if ($RetMode == 2) {return $dTSec;} // Raw, No rounding

   if ($RetMode == 3 or $RetMode == 4)
      {
       $NumSign = ($dTSec < 0)? '-':'+';
       $hours   = abs($dTSec/3600);
       $hh      = floor($hours);
       $minutes = 60*($hours - $hh);
       $mm      = floor($minutes);
       $ss      = round(60*($minutes - $mm), 0);
       if ($ss == 60){$mm++; $ss=0;}
       if ($mm == 60){$hh++; $mm=0;}
       $hh      = sprintf("%02d", $hh);
       $mm      = sprintf("%02d", $mm);
       $ss      = sprintf("%02d", $ss);

       if ($RetMode == 3)
          {$hhmmssDT = "$NumSign$hh:$mm:$ss";}
       else
          {$hhmmssDT = "$NumSign$hh"."h $mm"."m $ss"."s";}

       return $hhmmssDT;
      }

   if ($RetMode == 5) {return sprintf("%+1.15f", round($dTSec, 0) / 86400);}

   return $dTSec; // = FALSE on error, if year is out of valid range.

} // END OF  NASA_Delta_T_Poly (...)






/*
   Given a JDTT value and local time zone offset from UT,
   this function returns a table of  equivalent times in
   LT, UT and TT and the value of Delta T. This could be
   the Julian Date of some astronomical event.
*/

   function JDTT_to_Times_Table($JDTTArg, $TZhhmm='+00:00')
{
   $JDTT = floatval(trim($JDTTArg));

   $FracDayDT = NASA_Delta_T_Poly ($JDTT, 'G')/86400;
   $FracDayTZ = HMS_to_Hours(trim($TZhhmm))/24;

   $WETZ = ($FracDayTZ < 0)?'(West)':'(East)';
   if ($FracDayTZ == 0) {$WETZ = '      ';}

   $hhmmTZ   = substr(Hours_to_HMS(24*$FracDayTZ, 0, '+', ':'),0,6);
   $hhmmssDT = Hours_to_HMS(24*$FracDayDT, 0, '+', ':');

   $JDUT = $JDTT - $FracDayDT;
   $JDLT = $JDUT + $FracDayTZ;

   $DateTimeLT = Inv_JD($JDLT);
   $DateTimeUT = Inv_JD($JDUT);
   $DateTimeTT = Inv_JD($JDTT);

// Do some formatting.
   $FracDayDT = sprintf("%+1.8f", $FracDayDT);
   $FracDayTZ = sprintf("%+1.8f", $FracDayTZ);

   $JDLT = sprintf("%7.8f", $JDLT);
   $JDUT = sprintf("%7.8f", $JDUT);
   $JDTT = sprintf("%7.8f", $JDTT);

   $JDNumLT = floor($JDLT + 0.5);
   $JDNumUT = floor($JDUT + 0.5);
   $JDNumTT = floor($JDTT + 0.5);

   return
"------------------------------------------------------------------------------
    Julian Date                         Calendar Date      Time      JD Number
-------------------                    ---------------  -----------  ---------
$JDLT LT  Local Date/Time  $DateTimeLT LT   $JDNumLT
     $FracDayTZ TZ  Time Zone $WETZ                  $hhmmTZ:00 TZ
$JDUT UT  Universal Time   $DateTimeUT UT   $JDNumUT
     $FracDayDT DT  NASA Delta T                      $hhmmssDT DT
$JDTT TT  Terrestrial Time $DateTimeTT TT   $JDNumTT";
}

// *************************
// *************************

/*
   =========================================
   This function converts decimal days into
   a days, hours, minutes and seconds string
   in the format 'd hh mm ss'
   Example:
   367.10314836 returns '367d 02h 28m 32s'
*/
   function Days_to_DHMS ($Days, $ssDecimals=0)
{
   $days = FloatVal(trim($Days));
   $sign = ($days < 0)? '-':'';
   $days = abs($days);
   $d = 'd';

   $decimals = abs(IntVal(trim($ssDecimals)));
   if ($decimals > 3) {$decimals = 3;}

   $dd = floor($days);
   $hours = 24*($days - $dd);
   $HMS = Hours_to_HMS($hours, $decimals, '', 'h');

   return "$sign$dd$d  $HMS";
}








?>


